% This simulation example illustrates how the linking functions
% approach can be used. 

% Suppose we are interested in a simple confirmatory structural
% equation model with two latent variables  that are correlated 
% by 0.3 akin to the model used in the second simulation study 
% reported in the paper.
% First, the latent correlation matrix is defined:

phi=[1 0.3; ...
    0.3 1];

% Every latent variable should be measured by 4 indicator variables
% with loadings 0.6, 0.5, 0.4, and 0.3. Loadings are implemented
% as cell arrays in the algorithm. Loadings in the same cell indicate
% that variables have loadings on the same latent variable. We also
% define the number of manifest variables.

lambda=cell(2,1);
lambda{1}=[0.6 0.5 0.4 0.3]';
lambda{2}=[0.6 0.5 0.4 0.3]';
nMan=8;

% Now suppose we want one factor where the non-normality 
% is caused by the latent variables and one where non-normality 
% is caused by the errors. We need to choose non-linear functions 
% for the non-normal part and linear linking functions for the normal
% part. For illustration, we choose the non-linear functions used in
% the second simulation example.
% The functions for the latent and error variables, respectively,
% are stored in one cell array each. Each cell corresponds to one
% indicator variable specified above.

% latent linking functions
f=cell(nMan,1);
f{1}=@(xxx)(exp(xxx));
f{2}=@(xxx)(xrootsqrt(xxx));
f{3}=@(xxx)(xxx.^5+10*sin(xxx));
f{4}=@(xxx)(x_steps(xxx,50,3));
f{5}=@(xxx)(xxx);
f{6}=@(xxx)(xxx);
f{7}=@(xxx)(xxx);
f{8}=@(xxx)(xxx);

% error linking functions
g=cell(nMan,1);
for iii=1:4
    g{iii}=f{iii+4};
    g{iii+4}=f{iii};
end

% Functions can be defined locally (f_1, f_3) or as m-files (f_2, f_4). 
% The two m-files that belong to the functions need to reside in the same
% directory.
% Note: functions that are invariant under a point reflection in (0,0)
% will lead to symmetric distributions

% Now assume we also want to specify the kurtosis for all variables.
% We again define a cell as: 

spec_mom.goal=zeros(nMan,1);
spec_mom.order_moment=zeros(nMan,1);

% spec_mom.goal is a vector storing the target values for the desired 
% moments, spec_mom.order_moment denotes which moment we want to specify.
% We want the kurtosis to be 15 for all variables:

for iii=1:nMan
    spec_mom.goal(iii)=15;
    spec_mom.order_moment(iii)=4;
end

% We want to perform a simulation with 50, 150, 500, or 1000 observations 
% with 2000 samples each. 

N=[50 150 500 1000];
samples=2000;

% We call the function with an accuracy of a=7:

out_sim_ex=gendata( phi, lambda, N, f, g, 'acc', 7,...
    'samples', samples, 'spec_mom', spec_mom);

% The function creates the data in the working directory. If there are
% different sample sizes specified, as in this case, the function
% creates new directories in the working directory named N_SampleSize,
% so in this case N_50, N_150, etc.

% Now we provide an overview over all arguments and their defaults:
% Arguments without default:
% phi    - Latent correlation matrix 
% lambda - Cell array with loadings
% N      - Number or vector with number of observations
% f      - Functions for latent variables
% g      - Functions for error variables
% Arguments with a default value:
% 'acc'         - Accuracy for estimation (size of the p vector)
%               - Default: 6
% 'samples'     - Number of samples for each condition
%               - Default: 1
% 'spec_mom'    - Specifies the desired moment in the way described above
%               - Default: 0 (no moments will be approximated)
% 'acc2'        - Second accuracy value for the estimate of the error 
%                 correlations. Values larger than 3 might decrease the 
%                 speed of the algorithm significantly
%               - Default: 3 
% 'xi'          - A matrix with prespecified error correlations
%               - Default: uncorrelated errors (an identity matrix)
% 'file_name'   - A string containing the name of the data files. A number
%               - indicating the current number of simulations in this
%               - condition is added after the name.
%               - Default: '' (only the number will be used)
% 'folder_name' - A string containing the name of the folder where the
%               - data will be saved. If the directory does not exist,
%               - it will be created.
%               - Default: '' (no folder will be created)
% 'rmse'        - Indicates whether the rmse should be tracked (true/false). 
%               - Default: false

% The following characteristic values are saved (in the example) in the
% object out_sim_ex as additional output:
% errCorr - Resulting correlation matrix of the error variables
% vark    - The variance correction factors for each function
% cork    - The correlation correction factors for each function
% man     - Manifest variables created in the final sampling run
% lat     - The latent variables underlying the last manifest variables
% err     - The errors underlying the last manifest variables

